<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/
class MageBinary_BinaryPay_Helper_Payment extends Mage_Payment_Helper_Data
{
    /**
     * A valid duration to validate callback data
     * @var float
     */
    const VALID_TIME_DURATION = 3600 * 4 * 1000;

    /**
     * Check if callback data is valid
     * @param $callbackRawData
     * @throws BinaryPay_Exception
     * @throws Mage_Core_Exception
     */
    public function validateCallbackData($callbackRawData) {
        $callbackData = base64_decode($callbackRawData);
        $callbackData = Mage::getModel('core/encryption')->decrypt($callbackData);
        $callbackData = json_decode($callbackData, true);
        if (!$callbackData) {
            Mage::throwException("Invalid callback data!");
        }
        if (!is_array($callbackData) || !isset($callbackData['SID']) || !$callbackData['SID']) {
            Mage::throwException("Invalid callback data!");
        }
        // Validate timestamp
        if (!isset($callbackData['TS'])) {
            Mage::throwException("Invalid callback data!");
        }
        $callbackTimestamp = (float)$callbackData['TS'];
        if (microtime(true) - $callbackTimestamp > self::VALID_TIME_DURATION) {
            throw new BinaryPay_Exception("The callback data is out of time!");
        }
    }

    /**
     * Validate Signature 
     *
     * @return boolean
     */
    public function validateSignature($payload,$methodCode)
    {
        $payloadValidate = $payload;
        unset($payloadValidate['signature']);
        $salesStringStripped              = $this->stripJsonFromSalesString(json_encode($payloadValidate, JSON_UNESCAPED_SLASHES));
        $salesStringStrippedBase64encoded = $this->base64EncodeSalesString(trim($salesStringStripped));
        $signatureHash                    = $this->getSignatureHash(trim($salesStringStrippedBase64encoded),Mage::app()->getStore()->getId(),$methodCode);
        
        if($payload['reference'] != $this->_getReferenceNumber()){
            throw new BinaryPay_Exception(__('Invalid Reference'));
        }

        if($payload['signature'] !== $signatureHash) {
            throw new BinaryPay_Exception(__('Invalid Signature'));
        }

        return true;
    }

    /**
     * Strip out the json formatting, leaving only the name and values
     *
     * @param array $requestBody
     * @return mixed
     */
    public function stripJsonFromSalesString($requestBody)
    {
        $pattern = '/{"|":{"|","|":"|"},"|}],"|":|\[{"|"}}],"|}}|"}]"|},|,"|"}}|"}/';
        $replacement = '';

        $removeJsonFormatting = preg_replace($pattern, $replacement, $requestBody);
        $removeAllSpace = str_replace(' ', '', $removeJsonFormatting);
        $JSONStringWithoutFormatting = $removeAllSpace;

        return $JSONStringWithoutFormatting;
    }

    /**
     * Return a base 64 encoded string
     *
     * @param string $salesStringStripped
     * @return string
     */
    public function base64EncodeSalesString($salesStringStripped)
    {
        return base64_encode(str_replace(' ', '', $salesStringStripped));
    }

    /**
     * Get the signatiure hash for sending to latitude/GenoaPay
     *
     * @param string $salesStringStrippedBase64encoded
     * @param int $storeId
     * @param string $methodCode
     * @return string
     */
    public function getSignatureHash($salesStringStrippedBase64encoded,$storeId= null,$methodCode = null)
    {
        try {
            $clientSecret = $this->getPaymentMethodConfig($methodCode,'password', true);
            return hash_hmac('sha256', str_replace(' ', '', $salesStringStrippedBase64encoded), $clientSecret);
        } catch (\Exception $e) {
            throw $e;
        }
        return  null;
    }

    public function getPaymentMethodConfig($code,$field, $decrypt = false)
    {
        $data = Mage::getStoreConfig("payment/$code/$field", Mage::app()->getStore());
        if ($decrypt) {
            $data = Mage::helper('core')->decrypt($data);
        }
        return $data;
    }

    /**
     * Validate Total Amount call
     *
     * @return array
     */
    public function validateTotalAmount()
    {
        $quote = Mage::getModel('checkout/cart')->getQuote();
        $session = Mage::getSingleton('checkout/session');
        $totalAmount = (float)$quote->getGrandTotal();
        $currency = $quote->getQuoteCurrencyCode();
        $requestHash = md5(implode('||',[$totalAmount,$currency]));
        if($requestHash !== $session->getLatitudeTotalAmount()) {
            $session->unsLatitudeTotalAmount();
            Mage::throwException('Your cart was updated resulting in a price mismatch. We have marked your order as Pending for a review.');
        }
        $session->unsLatitudeTotalAmount();
        return true;
    }

    /**
     * _getReferenceNumber - get next order Id by last order id, to fix webpayment multiple increment id number bugs
     * @return Integer
     */
    protected function _getReferenceNumber()
    {
        $reserveOrderId = Mage::getSingleton('checkout/session')->getQuote()->reserveOrderId()->getReservedOrderId();
        if (!$reserveOrderId) {
            Mage::throwException('Can not get reserved order Id currently. Please try to checkout again.');
        }
        return $reserveOrderId;
    }
}
