<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/
require_once(Mage::getBaseDir('lib') . '/BinaryPay/autoload.php');
abstract class MageBinary_BinaryPay_Model_Abstract
{
    /**
     * code for payment registry
     * @var const string
     */
    protected $_code;

    public function getGateway()
    {
        $gateway = Mage::registry($this->_code);
        if (empty($gateway)) {
            $className = (isset(explode('_', $this->_code)[1])) ? ucfirst(explode('_', $this->_code)[1]) : ucfirst($this->_code);
            $gateway = BinaryPay::getGateway($className, $this->getCredentials(), $this->getPaymentMethodConfig('debug'));
            Mage::register($this->_code, $gateway);
        }
        if (empty($gateway)) {
            Mage::throwException('Magento cannot initialize Paymark API');
        }
        return $gateway;
    }

    public function getPaymentMethodConfig($field, $decrypt = false)
    {
        $code = $this->_code;
        $data = Mage::getStoreConfig("payment/$code/$field", Mage::app()->getStore());
        if ($decrypt) {
            $data = Mage::helper('core')->decrypt($data);
        }
        return $data;
    }

    /**
     * Retrieve PostPassword from database
     *
     * @return array
     */
    public function getCredentials()
    {
        return array(
            'username'      => $this->getPaymentMethodConfig('username', true),
            'password'      => $this->getPaymentMethodConfig('password', true),
            'environment'   => $this->getPaymentMethodConfig('sandbox'),
            'accountId'     => $this->getPaymentMethodConfig('account_id', true)
        );
    }

    public function generateTranscationParticular()
    {
        return 'MB'.substr(uniqid(rand()), 0, 16);
    }

    /**
     * check if the admin is login
     * @return boolean
     */
    public function isAdminLoggedIn()
    {
        return (Mage::getSingleton('admin/session')->isLoggedIn());
    }

    /**
     * check if the customer is login
     * @return boolean
     */
    public function isCustomerLoggedIn()
    {
        return Mage::getSingleton('customer/session')->isLoggedIn();
    }

    /**
     * Delete credit card token by their referece
     * @param  string $reference  the card token
     * @return boolean            tell if the card is deleted or not.
     */
    public function deleteTokenByReference($reference)
    {
        //check if current user session is login
        if (!$this->isAdminLoggedIn() && !$this->isCustomerLoggedIn()) {
            Mage::throwException('Please login to continue the operations.');
        }
        $customer = $this->getCustomer();

        //Get the tokens of the current customer
        $tokens     = $customer->getPmSavedTokens();

        if (empty($tokens) || !array_key_exists($reference, $tokens)) {
            Mage::throwException("Unable to find the object. $reference" . json_encode($tokens));
        }

        unset($tokens[$reference]);
        /* save the the reset to 'delete' */
        $customer->setPmSavedTokens($tokens)->save();
    }

    public function saveToken($data)
    {
        /* DATA must be Varien object */
        //check if current user session is login
        if (!$this->isAdminLoggedIn() && !$this->isCustomerLoggedIn()) {
            Mage::throwException('Please login to add your data.');
        }

        if (!($data instanceof Varien_Object)) {
            Mage::throwException('The saved object must be Varien Object.');
        }

        $customer = $this->getCustomer();

        /*The token backend will make sure this is array */
        $tokens   = $customer->getPmSavedTokens() !== null ? $customer->getPmSavedTokens() : array();

        //Do not save card if it has no reference.
        $tokenReference = $data->getTokenReference();
        if (empty($tokenReference)) {
            Mage::throwException('Need reference');
        }

        /* check the customers have existing cards */
        if (is_string($tokens) && !empty($tokens)) {
            $tokens = Mage::helper('core')->decrypt($customer->getPmSavedTokens());
            $tokens = Mage::helper('core')->jsonDecode($tokens);
            /* If nothings in the token list, make sure token is always array */
            $tokens = is_array($tokens) ? $tokens : array();
        }

        /*if the customer has no tokens or the card has not been saved , save card. Or return false*/
        if (!array_key_exists($tokenReference, $tokens)) {
            $tokens[$tokenReference] = $data->toArray();
            $customer->setPmSavedTokens($tokens)->save();
        }
    }

    /**
     * Get Current Paymark Allowed Card Type
     * @return array
     */
    public function getCurrentPaymarkAllowedCardType()
    {
        $paymarkCcTypes = $this->getPaymentMethodConfig('cctypes')?:array();
        if ($paymarkCcTypes) {
            $paymarkCcTypes = explode(',', $paymarkCcTypes);
        }
        return $paymarkCcTypes;
    }

    /**
     * Get Current Customer Save Tokens and cards info
     * @return Varien_Data_Collection
     */
    public function getCurrentCustomerTokens()
    {
        $customer   = $this->getCustomer();
        $collection = new Varien_Data_Collection();
        $tokens     = $customer->getPmSavedTokens();
        $storeCard =  $this->getPaymentMethodConfig('scard');


        //Check if the token saved card belong to current allowed card types setting
        // added additional check if StoreCard option is enabled SP-2567
        if (!empty($tokens) && $storeCard == 1 ) {
            foreach ($tokens as $cc) {
                if (isset($cc['card_type'])) {
                    $savedCcType = $this->translateToMagentoCcType($cc['card_type']);
                    $item = new Varien_Object;
                    $item->addData($cc);
                    $collection->addItem($item);
                }
            }
        }
        return $collection;
    }

    /**
     * Translate Magento Credit Card Code to Paymark's Code
     * @param  string $type
     * @return string
     */
    public function translateToPaymarkCcType($magentoCctype)
    {
        switch ($magentoCctype) {
            case 'VI':
            case 'Visa':
                $magentoCctype = MageBinary_BinaryPay_Model_System_General::CARD_TYPE_VISA;
                break;
            case 'DI':
            case 'DINERS_CLUB':
            case 'Diners Club':
                $magentoCctype = MageBinary_BinaryPay_Model_System_General::CARD_TYPE_DINERS;
                break;
            case 'AE':
            case 'AMERICAN_EXPRESS':
            case 'American Express':
                $magentoCctype = MageBinary_BinaryPay_Model_System_General::CARD_TYPE_AMEX;
                break;
            case 'MasterCard':
            case 'MASTERCARD':
                $magentoCctype = MageBinary_BinaryPay_Model_System_General::CARD_TYPE_MASTERCARD;
                break;
            case 'QCard':
                $magentoCctype = MageBinary_BinaryPay_Model_System_General::CARD_TYPE_QCARD;
                break;
        }
        return $magentoCctype;
    }


    /**
     * Translate Paymark Credit Card Code to Magento's Code
     * @param  string $type
     * @return string
     */
    public function translateToMagentoCcType($paymarkCctype)
    {
        switch ($paymarkCctype) {
            case MageBinary_BinaryPay_Model_System_General::CARD_TYPE_VISA:
            case MageBinary_BinaryPay_Model_System_General::CARD_VISA:
                $paymarkCctype = 'VI';
                break;
            case MageBinary_BinaryPay_Model_System_General::CARD_TYPE_DINERS:
            case MageBinary_BinaryPay_Model_System_General::CARD_TYPE_DINERS_CLUB:
            case MageBinary_BinaryPay_Model_System_General::CARD_DINERS:
                $paymarkCctype = 'DI';
                break;
            case MageBinary_BinaryPay_Model_System_General::CARD_TYPE_AMEX:
            case MageBinary_BinaryPay_Model_System_General::CARD_TYPE_AMERICAN_EXPRESS:
            case MageBinary_BinaryPay_Model_System_General::CARD_AMEX:
                $paymarkCctype = 'AE';
                break;
            case MageBinary_BinaryPay_Model_System_General::CARD_MASTERCARD:
                $paymarkCctype = 'MC';
                break;
            case MageBinary_BinaryPay_Model_System_General::CARD_QCARD:
                $paymarkCctype = 'QCARD';
                break;
        }
        return $paymarkCctype;
    }

    /**
     * Join Expiry month and expiry year in to one string for paymark
     * @param  string $expMonth
     * @param  string $expYear
     * @return string
     */
    public function getCardExpiryByMthAndYear($expMonth,$expYear)
    {
        return str_pad($expMonth, 2, '0', STR_PAD_LEFT) . substr($expYear, 2, 2);
    }

    /**
     * Give the current payment method code
     * @return string
     */
    public function getPaymentMethodCode()
    {
        return $this->_code;
    }


    public function getCardLogoUrlByCadeType($cardType)
    {
        $cardType = $this->translateToPaymarkCcType($cardType);
        return Mage::getModel('core/design_package')->getSkinUrl("images/paymark/" . Mage::getModel('magebinary_binarypay/system_general')->getCardLogoFileNameByCardType($cardType));
    }


    /**
     * Get allowed Credit Card types
     * @var array
     */
    public function getCcAvailableTypes()
    {
        //Paymark actived types
        //Get Current Paymark Allowed Card Type
        $paymarkAllowedCcTypes = $this->getCurrentPaymarkAllowedCardType();

        //Magento supported types
        $magentoAllCcTypes   = Mage::getModel('payment/config')->getCcTypes();
        $paymarkCcTypesNames = array();
        foreach ($paymarkAllowedCcTypes as $paymarkCcType) {
            $paymarkCcTypesNames[$paymarkCcType] = (array_key_exists($paymarkCcType, $magentoAllCcTypes))?
                $magentoAllCcTypes[$paymarkCcType]:$paymarkCcType;
        }
        return $paymarkCcTypesNames;
    }

    public function getCardNameByCardType($cardType)
    {
        //Translate to magento types
        $cardType = $this->translateToMagentoCcType($cardType);
        //Magento supported types
        $magentoAllCcTypes = Mage::getModel('payment/config')->getCcTypes();
        //if type does not exist, just return type
        if (array_key_exists($cardType, $magentoAllCcTypes)) {
            $cardName = $magentoAllCcTypes[$cardType];
        } else {
            $cardName = $cardType;
        }

        return $cardName;
    }

    /**
     * Display the current selected card logo
     * @return string
     */
    public function getDisplayCardLogoUrl()
    {
        //Get All logo Files to display
        $allowedCardTypes = $this->getCcAvailableTypes();
        $cardLogoUrls = array();
        foreach ($allowedCardTypes as $cardTypeCode => $cardTypeName) {
            $cardLogoUrls[$cardTypeCode]    = $this->getCardLogoUrlByCadeType($cardTypeCode);
        }
        return $cardLogoUrls;
    }

    /**
     * Check if now is adminhtml
     * @return boolean
     */
    public function isAdmin()
    {
        return Mage::app()->getStore()->isAdmin();
    }

    /**
     * Find current customer, for both adminhtml and front end.
     * @return Object current customer
     */
    public function getCustomer()
    {
            //To find current customer, try use register first, for adminhtml customer
            $customer = Mage::registry('current_customer');

            //For adminhtml quote/order, register can not find the customer, so use followinggetSingleton
            if ($this->isAdmin() && !isset($customer)) {
                $customer = Mage::getSingleton('adminhtml/session_quote')->getCustomer();
            }

            //Still not set, then use helper. Mainly for fronthtml.
            if (empty($customer)) {
                $customer = Mage::helper('customer')->getCustomer();
            }
            return $customer;
    }
}
