<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/
class MageBinary_BinaryPay_Model_Method_Directpost extends Mage_Payment_Model_Method_Cc
{
    protected $_formBlockType               = 'magebinary_binarypay/directpost_form';
    protected $_infoBlockType               = 'magebinary_binarypay/directpost_info';
    protected $_code                        = MageBinary_BinaryPay_Model_Config::PAYMARK_DIRECTPOST;
    protected $_isGateway                   = true;
    protected $_canAuthorize                = true;
    protected $_canCapture                  = true;
    protected $_canRefund                   = true;
    protected $_canRefundInvoicePartial     = true;
    protected $_canFetchTransactionInfo     = true;
    protected $_canUseCheckout              = true;
    protected $_canVoid                     = true;

    const SUCCESSFUL = 'SUCCESSFUL';
    const DECLINED = 'DECLINED';
    const CANCELLED = 'CANCELLED';
    const INPROGRESS = 'INPROGRESS';
    const UNKNOWN = 'UNKNOWN';

    /**
     * Check whether there are CC types set in configuration
     *
     * @param Mage_Sales_Model_Quote|null $quote
     * @return bool
     */

    public function isAvailable($quote = null)
    {
        return  Mage::getModel('magebinary_binarypay/directpost')->getPaymentMethodConfig('active')
            && parent::isAvailable($quote);
    }

    public function assignData($data)
    {
        if (!($data instanceof Varien_Object)) {
            $data = new Varien_Object($data);
        }

        $info = $this->getInfoInstance();
        /* Check whether the save cc option was checked */
        $ccSaveCard = $data->getData('cc_save_card');
        $info->setCcSaveCard(empty($ccSaveCard) ? 0 : 1);

        $cardToken = $data->getData('card_token');
        $info->setCardToken(empty($cardToken) ? null : $cardToken);
        $info->setPoNumber($data->getPurchaseOrder());

        return parent::assignData($data);
    }

    public function validate()
    {
        $info = $this->getInfoInstance();
        /*TODO: validates as numbers and digits*/
        if ($info->getCardToken()) {
            return $this;
        }

        return parent::validate();
    }

    public function capture(Varien_Object $payment, $amount)
    {
        $code           = explode('_', $this->_code);
        $gatewayName    = next($code);
        $api            = Mage::getModel("magebinary_binarypay/$gatewayName");
        $gateway        = $api->getGateway();
        $order          = $payment->getOrder();
        $customer       = $order->getCustomer();
        $helper         = Mage::helper('core');
        $session        = Mage::getSingleton('checkout/session');
        /* Card Processing */
        $cardInfo       = array(
            BinaryPay::CARD_NUMBER => $payment->getCcNumber(),
            BinaryPay::CARD_EXPIRY => $api->getCardExpiryByMthAndYear(
                $payment->getCcExpMonth(),
                $payment->getCcExpYear()
            )
        );

        if ($payment->getCardToken()) {
            $card = array("cardToken" => Mage::helper('core')->decrypt($payment->getCardToken()));
        } else {
            $card = array(
                BinaryPay::CARD_NUMBER   => $payment->getCcNumber(),
                BinaryPay::CARD_TYPE     => $api->translateToPaymarkCcType($payment->getCcType()),
                BinaryPay::CARD_EXPIRY   => $api->getCardExpiryByMthAndYear(
                    $payment->getCcExpMonth(),
                    $payment->getCcExpYear()
                ),
                BinaryPay::CARD_HOLDER   => $this->getOwner($payment),
                BinaryPay::CARD_CSC      => $payment->getCcCid(),
                BinaryPay::STORE_CARD    => $payment->getCcSaveCard()
                );
        }

        $email = $this->getPaymentMethodConfig('email_notify');

        /* Transcation Processing */
        $transaction = array(
            BinaryPay::ACCOUNTID        => $api->getPaymentMethodConfig('account_id', true),
            BinaryPay::AMOUNT           => $amount,
            BinaryPay::REFERENCE        => $order->getIncrementId(),
            BinaryPay::PARTICULAR       => $api->generateTranscationParticular(),
            BinaryPay::EMAIL            => ($email) ? $email : '',
            BinaryPay::CARD_TOKEN_REF   => $api->generateCreditCardTokenReference($cardInfo),
        );

        try {
            $response = $gateway->purchase(array_merge($transaction, $card));

            /* Save token for the customers only if not paid by token :) */
            if ($payment->getCcSaveCard() && !$payment->getCardToken()) {
                $api->saveToken($response);
            }

            $poNumber = $payment->getPoNumber();
            if (!empty($poNumber)) {
                $response = array_merge(
                    $response,
                    array('purchaseOrder' => $payment->getPoNumber())
                );
            }

            $payment->setStatus(self::STATUS_APPROVED)
                    ->setLastTransId($response['transactionId'])
                    ->setTransactionId($response['transactionId']);

            $response = $gateway->clearify($response);
            $payment->setAdditionalData(serialize($response));
        } catch (BinaryPay_Exception $e) {
            Mage::throwException($e->getMessage());
        } catch (Exception $e) {
            Mage::throwException('Something massively went wrong. Please try again. If the problem still exists, please contact us');
            Mage::log($e->getTrace(), null, 'BinaryPay.log', true);
        }

        return $this;
    }

    /**
     * Refund specified amount for payment
     *
     * @param Varien_Object $payment
     * @param float $amount
     *
     * @return Mage_Payment_Model_Abstract
     */
    public function refund(Varien_Object $payment, $amount)
    {
        $api        = Mage::getModel('magebinary_binarypay/directpost');
        $gateway    = $api->getGateway();

        $order      = $payment->getOrder();
        $refund     = array(
             BinaryPay::ACCOUNTID               => $api->getPaymentMethodConfig('account_id', true),
             BinaryPay::TRANSACTION_ID          => $payment->getParentTransactionId(),
             BinaryPay::AMOUNT                  => $amount,
             BinaryPay::REFERENCE               => $order->getIncrementId(),
             BinaryPay::PARTICULAR              => $order->getIncrementId(),
             BinaryPay::EMAIL                   => $order->getCustomerEmail()
        );

        try {
            $refundTranscation = $gateway->refund($refund);
            $payment->setTransactionId($refundTranscation['transactionId']);
            $shouldCloseCaptureTransaction = $payment->getOrder()->canCreditmemo() ? 0 : 1;
            $payment->setShouldCloseParentTransaction($shouldCloseCaptureTransaction)
                    ->setTransactionAdditionalInfo(serialize($refundTranscation), $payment->getTransactionId());
        } catch (BinaryPay_Exception $e) {
            Mage::throwException($e->getMessage());
        } catch (Exception $e) {
            Mage::throwException('Something massively went wrong. Please try again. If the problem still exists, please contact us');
            Mage::log($e->getTrace(), null, 'paymark.log', true);
        }

        return $this;
    }

    public function fetchTransactionInfo(Mage_Payment_Model_Info $payment, $transactionId)
    {
        $api        = Mage::getModel('magebinary_binarypay/directpost');
        $gateway    = $api->getGateway();
        try {
            $transcation = $gateway->getTransaction($transactionId);
        } catch (BinaryPay_Exception $e) {
            Mage::throwException($e->getMessage());
        } catch (Exception $e) {
            Mage::throwException('Something massively went wrong. Please try again. If the problem still exists, please contact us');
            Mage::log($e->getTrace(), null, 'paymark.log', true);
        }
        return $transcation;
    }


    protected function getOwner($payment)
    {
        return htmlspecialchars(trim($payment->getCcOwner()), ENT_QUOTES, 'UTF-8');
    }
}