<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 - 2019 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 - 2019 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/
class MageBinary_BinaryPay_Model_Method_Genoapay extends MageBinary_BinaryPay_Model_Method_Abstract
{
    protected $_formBlockType               = 'magebinary_binarypay/genoapay_form';
    protected $_infoBlockType               = 'magebinary_binarypay/genoapay_info';
    protected $_code                        = MageBinary_BinaryPay_Model_Config::GENOAPAY;
    protected $_modelName                   = 'magebinary_binarypay/genoapay';
    protected $_returnUrl                   = 'binarypay/genoapay/return';
    protected $_isGateway                   = true;
    protected $_canAuthorize                = true;
    protected $_canCapture                  = true;
    protected $_canRefund                   = true;
    protected $_canRefundInvoicePartial     = true;
    protected $_canFetchTransactionInfo     = true;
    protected $_canUseCheckout              = true;
    protected $_canVoid                     = true;

    const DEFAULT_VALUE                     = 'NONE';
    const SUCCESSFUL                        = 'COMPLETED';
    const DECLINED                          = 'DECLINED';
    const CANCELLED                         = 'CANCELLED';
    const INPROGRESS                        = 'INPROGRESS';
    const UNKNOWN                           = 'UNKNOWN';
    const FAILED                            = 'FAILED';
    const GENOAPAY_ENABLED                  = "payment/genoapay/active";
    const GENOAPAY_CURRENCY                 = "payment/genoapay/currency";

    /**
     * getCheckoutRedirectUrl
     * @return string
     */
    public function getCheckoutRedirectUrl()
    {
        $purchaseUrl    = $this->getPurchaseUrl();
        return $purchaseUrl;
    }

    /**
     * assignData
     * @param  Varien_Object | Array $data
     * @return MageBinary_BinaryPay_Model_Method_Genopay
     */
    public function assignData($data)
    {
        $session = Mage::getSingleton('checkout/session');

        if (!($data instanceof Varien_Object)) {
            $data = new Varien_Object($data);
        }

        $this->getInfoInstance()
             ->setPoNumber($data->getPurchaseOrder());

        return $this;
    }

    /**
     * Capture payment
     *
     * @param Varien_Object $payment
     * @param float $amount
     *
     * @return MageBinary_BinaryPay_Model_Method_Genoapay
     */
    public function capture(Varien_Object $payment, $amount)
    {
        $session = Mage::getSingleton('checkout/session');

        $gateway    = Mage::getModel($this->_modelName)->getGateway();
        $response   = $gateway->clearify($session->getResponse());

        $payment->setLastTransId($response['token'])
                ->setTransactionId($response['token'])
                ->setAdditionalData(serialize($response));
        return $this;
    }

    /**
     * Refund specified amount for payment
     *
     * @param Varien_Object $payment
     * @param float $amount
     *
     * @return Mage_Payment_Model_Abstract
     */
    public function refund(Varien_Object $payment, $amount)
    {
        $api        = Mage::getModel($this->_modelName);
        $gateway    = $api->getGateway();

        $order      = $payment->getOrder();
        $refund     = array(
             BinaryPay::PURCHASE_TOKEN          => $payment->getParentTransactionId(),
             BinaryPay::CURRENCY                => $this->getCurrencyCode(),
             BinaryPay::AMOUNT                  => $amount,
             BinaryPay::REFERENCE               => $order->getIncrementId(),
             BinaryPay::REASON                  => '',
        );

        try {
            $response = $gateway->refund($refund);
            // validate response
            // {"status":200,"body":"{\"refundId\":\"17e10294-c64d-42b3-ab08-51ebd726d7d2\",\"refundDate\":\"2019-05-21T12:33:44+12:00\",\"reference\":\"G1117-497\"}"}

            $payment->setTransactionId($response['refundId']);
            $shouldCloseCaptureTransaction = $payment->getOrder()->canCreditmemo() ? 0 : 1;
            $payment->setShouldCloseParentTransaction($shouldCloseCaptureTransaction)
                    ->setTransactionAdditionalInfo(serialize($response), $payment->getTransactionId());
        } catch (BinaryPay_Exception $e) {
            Mage::throwException($e->getMessage());
        } catch (Exception $e) {
            Mage::throwException('Something massively went wrong. Please try again. If the problem still exists, please contact us');
            Mage::log($e->getTrace(), null, 'paymark.log', true);
        }

        return $this;
    }

    /**
     * Fetch transaction info
     *
     * @param Mage_Payment_Model_Info $payment
     * @param string $transactionId
     * @return array
     */
    public function fetchTransactionInfo(Mage_Payment_Model_Info $payment, $transactionId)
    {
        $api        = Mage::getModel($this->_modelName);
        $gateway    = $api->getGateway();

        try {
            $transcation = $gateway->retrieve(array(
                BinaryPay::PURCHASE_TOKEN => $transactionId
            ));
        } catch (BinaryPay_Exception $e) {
            Mage::throwException($e->getMessage());
        } catch (Exception $e) {
            Mage::throwException('Something massively went wrong. Please try again. If the problem still exists, please contact us');
            Mage::log($e->getTrace(), null, 'BinaryPay.log', true);
        }
        return $transcation;
    }

    public function getPurchaseUrl()
    {
        try {
            /**
             * TODO: Support backend payment
             * @var boolean
             */
            $isBackend  = false;

            $api        = Mage::getModel($this->_modelName);
            $gateway    = $api->getGateway();

            $reference          = $this->_getReferenceNumber();
            $amount             = $this->getAmount($isBackend);

            $billingAddress        = $this->_getQuote()->getBillingAddress();
            $shippingAddress    = $this->_getQuote()->getShippingAddress();

            $payment = array(
                BinaryPay::REFERENCE                => (string) $reference,
                BinaryPay::AMOUNT                   => round($amount, 2),
                BinaryPay::CURRENCY                 => $this->getCurrencyCode() ?: self::DEFAULT_VALUE,
                BinaryPay::RETURN_URL               => Mage::getUrl($this->_returnUrl, array('callback_data' => $this->_generateCallbackData())),
                BinaryPay::MOBILENUMBER             => $billingAddress->getTelephone() ?: '0210123456',
                BinaryPay::EMAIL                    => $this->_getCustomerEmail() ?: self::DEFAULT_VALUE,
                BinaryPay::FIRSTNAME                => $this->_getFirstName() ?: $billingAddress->getFirstname(),
                BinaryPay::SURNAME                  => $this->_getSurname() ?: $billingAddress->getLastname(),
                BinaryPay::SHIPPING_ADDRESS         => current($shippingAddress->getStreet()) ?: self::DEFAULT_VALUE,
                BinaryPay::SHIPPING_COUNTRY_CODE    => $shippingAddress->getCountryId() ?: self::DEFAULT_VALUE,
                BinaryPay::SHIPPING_POSTCODE        => $shippingAddress->getPostcode() ?: self::DEFAULT_VALUE,
                BinaryPay::SHIPPING_SUBURB          => $shippingAddress->getRegion() ?: self::DEFAULT_VALUE,
                BinaryPay::SHIPPING_CITY            => $shippingAddress->getCity() ?: self::DEFAULT_VALUE,
                BinaryPay::BILLING_ADDRESS          => current($billingAddress->getStreet()) ?: self::DEFAULT_VALUE,
                BinaryPay::BILLING_COUNTRY_CODE     => $billingAddress->getCountryId() ?: self::DEFAULT_VALUE,
                BinaryPay::BILLING_POSTCODE         => $billingAddress->getPostcode() ?: self::DEFAULT_VALUE,
                BinaryPay::BILLING_SUBURB           => $billingAddress->getRegion() ?: self::DEFAULT_VALUE,
                BinaryPay::BILLING_CITY             => $billingAddress->getCity() ?: self::DEFAULT_VALUE,
                BinaryPay::TAX_AMOUNT               => round($shippingAddress->getQuote()->getShippingAddress()->getData('tax_amount'), 2),
                BinaryPay::PRODUCTS                 => $this->_getQuoteProducts(),
                BinaryPay::SHIPPING_LINES           => [
                    $this->_getShippingData()
                ]
            );

            $response   = $gateway->purchase($payment);

            $this->setSesionReferense($reference, $response);

            $this->setSessionTotalAmount($this->_getQuote()->getGrandTotal(),$this->_getQuote()->getQuoteCurrencyCode());

            $purchaseUrl = $this->_getPurchaseUrl($response);
        } catch (BinaryPay_Exception $e) {
            Mage::throwException($e->getMessage());
        } catch (Exception $e) {
            $message = $e->getMessage() ?: 'Something massively went wrong. Please try again. If the problem still exists, please contact us';
            Mage::throwException($message);
            Mage::log($e->getTrace(), null, 'BinaryPay.log', true);
        }

        return $purchaseUrl;
    }

    public function setSesionReferense($reference, $response)
    {
        Mage::getSingleton('checkout/session')->setGenoapayReference($reference);
        Mage::getSingleton('checkout/session')->setGenoapayToken($response['token']);
    }

    public function setSessionTotalAmount($totalAmount,$currency)
    {
        $requestHash = md5(implode('||',[$totalAmount,$currency]));
        Mage::getSingleton('checkout/session')->setLatitudeTotalAmount($requestHash);
    }

    /**
     * _getPurchaseUrl
     * @param  array $purchase
     * @return string
     */
    private function _getPurchaseUrl($purchase)
    {
        // Get Purchase Url from response
        if (isset($purchase['paymentUrl'])) {
            $purchaseUrl = $purchase['paymentUrl'];
        } else {
            Mage::throwException('Oops! Some errors has occurred, we can not redirect you to the GenoaPay. Please try again later.');
        }
        return $purchaseUrl;
    }

    protected function _getCustomerEmail()
    {
        return ($this->isLoggedIn()) ? $this->getCustomer()->getEmail() : $this->_getQuote()->getCustomerEmail();
    }

    public function _getFirstName()
    {
        return ($this->isLoggedIn()) ? $this->getCustomer()->getFirstname() : '';
    }

    public function _getSurname()
    {
        return ($this->isLoggedIn()) ? $this->getCustomer()->getLastname() : '';
    }

    private function _generateCallbackData() {
        $callbackData = array(
            'SID'   => Mage::getSingleton('core/session')->getEncryptedSessionId(),
            'QID'   => $this->_getQuote()->getId(),
            'CID'   => Mage::getSingleton('customer/session')->getCustomerId(),
            'VID'   => Mage::getModel('core/session')->getVisitorId(),
            'SCID'  => Mage::getSingleton('core/cookie')->get('frontend_cid'),
            'TS'    => microtime(true)
        );
        return base64_encode(Mage::getModel('core/encryption')->encrypt(json_encode($callbackData)));
    }

    /**
     * Check whether payment method is applicable to quote
     * Purposed to allow use in controllers some logic that was implemented in blocks only before
     *
     * @param Mage_Sales_Model_Quote $quote
     * @param int|null $checksBitMask
     * @return bool
     */
    public function isApplicableToQuote($quote, $checksBitMask)
    {
        return $this->isAvailable($quote) && Mage::getStoreConfig(self::GENOAPAY_ENABLED, $store) && (Mage::getStoreConfig(self::GENOAPAY_CURRENCY) == Mage::app()->getStore()->getCurrentCurrencyCode());
    }
}