<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/

class MageBinary_BinaryPay_Model_Quote_Address_Total_Surcharge extends Mage_Sales_Model_Quote_Address_Total_Abstract
{
    const MAGEBINARY_BINARYPAY_SURCHARGE_ADDRESS_TAX_ADDED = 'magebinary_binarypay_surcharge_address_tax_added';
    const BINARY_PAY_GATEWAY = array(
        MageBinary_BinaryPay_Model_Config::PAYMARK_DIRECTPOST,
        MageBinary_BinaryPay_Model_Config::PAYMARK_WEBPAYMENT,
        MageBinary_BinaryPay_Model_Config::PAYMARK_ONLINEEFTPOS,
        MageBinary_BinaryPay_Model_Config::QCARD_FLEXILONG
    );
    const SURCHARGE_MIN_AMOUNT = 1;

    protected $_code = 'binarypay_surcharge';

    protected function _resetSurcharge(Mage_Sales_Model_Quote_Address $address)
    {
        $address->setBinarypaySurchargeAmount(0);
        $address->setBaseBinarypaySurchargeAmount(0);
        $address->setBinarypaySurchargeTaxAmount(0);
        $address->setBaseBinarypaySurchargeTaxAmount(0);
    }

    /**
     * Determine if we should apply the surcharge
     *
     * @param Mage_Sales_Model_Quote_Address $address
     *
     * @return bool
     */
    protected function _init(Mage_Sales_Model_Quote_Address $address)
    {
        if (!$this->_canApply($address)) {
            return false;
        }

        if (!is_numeric($address->getQuote()->getId())) {
            return false;
        }

        // We can return if there are no items
        $items = $address->getAllItems();
        if (!count($items)) {
            return false;
        }

        return true;
    }

    /**
     * the auto group assign can leave the customer group id in a wrong state
     * fix it here
     *
     * @param Mage_Sales_Model_Quote $quote
     */
    protected function _fixQuote(Mage_Sales_Model_Quote $quote)
    {
        //Magento bug? $quote->getCustomerGroupId() does not supply the correct group
        //reset the data here by loading it directly from the db
        if ($quote->getCustomerId()) {
            $customer = Mage::getModel('customer/customer')->load($quote->getCustomerId());
            if ($customer->getId()) {
                $customerGroupId = $customer->getGroupId();
                if ($customerGroupId) {
                    $quote->setData('customer_group_id', $customerGroupId);
                }
            }
        }
        return $this;
    }

    public function collect(Mage_Sales_Model_Quote_Address $address)
    {
        $this->_resetSurcharge($address);

        if (!$this->_init($address)) {
            return $this;
        }

        $quote = $address->getQuote();
        $this->_fixQuote($quote);

        // Surcharge
        $surcharge = $this->_initSurcharge($address, $quote);

        // Update address
        $this->_updateAddressWithSurcharge($address, $surcharge);

        //  Update quote
        $this->_updateQuoteWithSurcharge($quote, $surcharge);

        return $this;
    }

    /**
     * prepare the surcharge for display in the totals list
     *
     * @param Mage_Sales_Model_Quote_Address $address
     *
     * @return $this|array
     */
    public function fetch(Mage_Sales_Model_Quote_Address $address)
    {
        $amount = $address->getBinarypaySurchargeAmount();
        if ($amount != 0) {
            $address->addTotal(
                array(
                     'code'  => $this->getCode(),
                     'title' => MageBinary_BinaryPay_Model_Surcharge::BINARYPAY_TITLE,
                     'value' => $amount
                )
            );
        }
        return $this;
    }

    /**
     * create a new surcharge model to collect amounts
     * and set common values for calculation
     *
     * @param Mage_Sales_Model_Quote_Address $address
     * @param Mage_Sales_Model_Quote         $quote
     *
     * @return MageBinary_BinaryPay_Model_Surcharge
     */
    protected function _initSurcharge(Mage_Sales_Model_Quote_Address $address, Mage_Sales_Model_Quote $quote)
    {
        $surcharge = Mage::getModel('magebinary_binarypay/surcharge');
        $surcharge->setStoreId($quote->getStoreId());
        $surcharge->setAmount($this->_getBaseSurcharge($address, $quote));
        $surcharge->setBaseAmount($this->_getBaseSurcharge($address, $quote));

        $surcharge = $this->_calculateSurchargeTax($address, $surcharge);
        return $surcharge;
    }

    protected function _getBaseSurcharge($address, $quote)
    {
        // Get surcharge from BinaryPay Api
        $surcharge = 0.5;

        // Get currrent payment method
        $paymentMethod  = $quote->getPayment()->getMethod();

        $paymentMethod  = explode('_', $paymentMethod);
        $paymentMethod  = next($paymentMethod);

        // Only set surcharge after user selected the payment method
        if ($paymentMethod) {
            // Get store owner's surchage
            $customSurcharge = Mage::getModel("magebinary_binarypay/$paymentMethod")->getPaymentMethodConfig('surcharge');

            if (strpos($customSurcharge, '%')) {
                $customSurcharge = explode('%', $customSurcharge);
                $customSurcharge = current($customSurcharge);
                $customSurcharge = $address->getSubtotal() * $customSurcharge * 0.01;
            }

            if (is_float($customSurcharge) || is_numeric($customSurcharge)) {
                $surcharge += $customSurcharge;
            }

            return $surcharge;
        }
    }


    /**
     * update the address object with the applicable surcharge and update address tax and grand total
     *
     * @param Mage_Sales_Model_Quote_Address   $address
     * @param MageBinary_BinaryPay_Model_Surcharge $surcharge
     */
    protected function _updateAddressWithSurcharge(Mage_Sales_Model_Quote_Address $address,
        MageBinary_BinaryPay_Model_Surcharge $surcharge )
    {
        //apply surcharge amounts in order currency
        $address->setBinarypaySurchargeAmount($surcharge->getAmount());
        $address->setBaseBinarypaySurchargeAmount($surcharge->getBaseAmount());

        // Update surcharge Tax amount
        $address->setTaxAmount($address->getTaxAmount() + $surcharge->getTaxAmount());
        $address->setBaseTaxAmount($address->getBaseTaxAmount() + $surcharge->getBaseTaxAmount());

        $appliedTaxes = $address->getAppliedTaxes();
        $currentTaxes = current($appliedTaxes);

        // TODO: Add support to Multi tax for single area
        if (isset($currentTaxes['amount']) && isset($currentTaxes['base_amount'])) {
            $currentTaxes['amount']         += $surcharge->getTaxAmount();
            $currentTaxes['base_amount']    += $surcharge->getBaseTaxAmount();
        }

        if ($appliedTaxes) {
            reset($appliedTaxes);
            $firstKey  = key($appliedTaxes);
            $appliedTaxes[$firstKey]    = $currentTaxes;
        }

        $address->setAppliedTaxes($appliedTaxes);

        $address->setGrandTotal($address->getGrandTotal() + $surcharge->getAmount() + $surcharge->getTaxAmount());
        $address->setBaseGrandTotal(
            $address->getBaseGrandTotal() + $surcharge->getBaseAmount() + $surcharge->getBaseTaxAmount()
        );

        return $this;
    }

    /**
     * update quote object with surcharge amounts
     *
     * @param Mage_Sales_Model_Quote           $quote
     * @param MageBinary_BinaryPay_Model_Surcharge $surcharge
     */
    protected function _updateQuoteWithSurcharge(Mage_Sales_Model_Quote $quote,
        MageBinary_BinaryPay_Model_Surcharge $surcharge)
    {
        $quote->setBinarypaySurchargeAmount((float)$quote->getBinarypaySurchargeAmount() + $surcharge->getAmount());
        $quote->setBaseBinarypaySurchargeAmount(
            (float)$quote->getBaseBinarypaySurchargeAmount() + $surcharge->getBaseAmount()
        );

        return $this;
    }

    protected function _calculateSurchargeTax(Mage_Sales_Model_Quote_Address $address,
        MageBinary_BinaryPay_Model_Surcharge $surcharge)
    {
        $quote      = $address->getQuote();
        $storeId    = $surcharge->getStoreId();
        $taxRate    = $this->_getSurchargeTaxRate();

        $surcharge->setBaseTaxAmount(
            $this->_roundTax(
                $address,
                $surcharge->getBaseAmount() * ($taxRate / 100),
                $taxRate,
                true,
                'base'
            )
        );

        $surcharge->setTaxAmount(
            $this->_roundTax(
                $address,
                $surcharge->getAmount() * ($taxRate / 100),
                $taxRate,
                true,
                'regular'
            )
        );

        if (!Mage::registry(self::MAGEBINARY_BINARYPAY_SURCHARGE_ADDRESS_TAX_ADDED)) {
            $taxCalculationModel = Mage::getSingleton('tax/calculation');
            $request = $taxCalculationModel->getRateRequest(
                $quote->getShippingAddress(),
                $quote->getBillingAddress(),
                $quote->getCustomerTaxClassId(),
                $storeId
            );
            $request->setStore($quote->getStoreId());
            Mage::getModel('magebinary_binarypay/quote_address_total_tax')->_saveAppliedTaxes(
                $address,
                $taxCalculationModel->getAppliedRates($request),
                $surcharge->getTaxAmount(),
                $surcharge->getBaseTaxAmount(),
                $taxRate
            );
            Mage::register(self::MAGEBINARY_BINARYPAY_SURCHARGE_ADDRESS_TAX_ADDED, true);
        }

        return $surcharge;
    }

    protected function _getSurchargeTaxRate()
    {
        $quote      = Mage::getSingleton('checkout/session')->getQuote();
        $country    = $quote->getBillingAddress()->getCountryId();

        $taxRateCollection = Mage::getResourceModel('tax/calculation_rate_collection');
        $taxRateCollection->addFieldToFilter('tax_country_id', $country);

        /* TODO: Add support to multiple tax for same area*/
        $taxRate = $taxRateCollection->getFirstItem()->getData('rate');
        // foreach ($taxRateCollection as $rate) {
            // $taxRate = $rate->getData('rate');
        // }

        return $taxRate;
    }

    /**
     * round tax with existing rounding tax deltas
     *
     * @param        $address
     * @param        $tax
     * @param        $rate
     * @param        $direction
     * @param string $type
     *
     * @return float
     */
    protected function _roundTax($address, $tax, $rate, $direction, $type = 'regular')
    {
        if ($tax != 0) {
            $delta = 0;
            switch (Mage::getSingleton('tax/config')->getAlgorithm(Mage::app()->getStore()->getId())) {
                case Mage_Tax_Model_Calculation::CALC_TOTAL_BASE:
                    $deltas = $address->getRoundingDeltas();
                    $key = $type . $direction;
                    $delta = isset($deltas[$key][$rate]) ? $deltas[$key][$rate] : 0;
                    //no break;
                default:
                    return $this->_round(Mage::getSingleton('tax/calculation')->round($tax + $delta));
                    break;
            }
        }
    }

    protected function _canApply(Mage_Sales_Model_Quote_Address $address)
    {
        $quote          = $address->getQuote();
        $paymentMethod  = $quote->getPayment()->getMethod();

        if (!isset($paymentMethod)) {
            return false;
        }

        if (in_array($paymentMethod, self::BINARY_PAY_GATEWAY)) {
            $postfix  = explode('_', $paymentMethod);
            $postfix  = next($postfix);
            if (!Mage::getModel("magebinary_binarypay/$postfix")->getPaymentMethodConfig('surcharge_active')) {
                return false;
            }
        }

        if ($paymentMethod != null && !in_array($paymentMethod, self::BINARY_PAY_GATEWAY)) {
            return false;
        }

        $totalAmount = $address->getSubtotal();

        if ($totalAmount < self::SURCHARGE_MIN_AMOUNT) {
            return false;
        }


        return true;
    }

    // protected function _getAllBinaryPayPayment()
    // {
    //     $payments = Mage::getSingleton('payment/config')->getActiveMethods();
    //     $methods = array(
    //             array(
    //                 'value' => '',
    //                 'label' => Mage::helper('adminhtml')->__('--Please Select--')),
    //     );

    //     foreach ($payments as $paymentCode => $paymentModel) {
    //         $paymentTitle = Mage::getStoreConfig('payment/'.$paymentCode.'/title');
    //         $methods[$paymentCode] = array(
    //             'label'     => $paymentTitle,
    //             'value'     => $paymentCode,
    //         );
    //     }

    //     return $methods;
    // }

    /**
     * round amount according to Magento's current rounding setting
     *
     * @param $amount
     *
     * @return float
     */
    protected function _round($amount)
    {
        return Mage::app()->getStore()->roundPrice($amount);
    }
}