<?php
/**
 * Magento BinaryPay Payment Extension
 *
 * NOTICE OF LICENSE
 *
 * Copyright 2017 MageBinary
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @category    MageBinary
 * @package     MageBinary_BinaryPay
 * @author      MageBinary Team
 * @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
 * @license     http://www.apache.org/licenses/LICENSE-2.0
 */
class MageBinary_BinaryPay_EftposController extends Mage_Core_Controller_Front_Action
{
    /**
     * $_checkoutSession - checkout session model
     */
    private $_checkoutSession;

    public function preDispatch()
    {
        parent::preDispatch();
        $session        = Mage::getSingleton('core/session');
        $incrementId    = $session->getIncrementId();

        if (!Mage::getModel('magebinary_binarypay/onlineeftpos')->getPaymentMethodConfig('active') ||
            empty($incrementId)) {
            return $this->_redirect('/');
        }
    }

    protected function _validateCallback()
    {
        $request            = $this->getRequest();
        $merchantOrderId    = $request->getParam('merchantOrderId');
        $status             = $request->getParam('status');
        $response           = $request->getPost();

        if (empty($request->getPost())) {
            $message = $this->__('Unknown response from Paymark');
            Mage::throwException($message);
        }

        $validateKeys = array(
            'merchantOrderId',
            'status',
            'transactionId',
            'signature',
        );

        foreach ($validateKeys as $key) {
            if (!array_key_exists($key, $response)) {
                $message = 'Unexpeted response: $response';
                break;
            }
        }

        // If response's statu is not been set
        if (empty($status)) {
            $message = "Unknown status";
        }

        // Response doesnt contain an order id
        if (!isset($merchantOrderId)) {
            $message = 'Unknown order status';
        }

        if (isset($message)) {
            Mage::throwException($this->__($message));
        }

        return $this;
    }

    /**
     * _processOrder - process order and update transaction data information by retrieve paymark's Api response
     * @param  array $response - Response after OE purchase
     * @return $this
     */
    protected function _processOrder($response)
    {
        $session    = Mage::getSingleton('core/session');
        $order      = Mage::getModel('sales/order')->load($response['merchantOrderId'], 'increment_id');

        // Fetch Local Transaction Data
        $transactionData = unserialize($order->getPayment()->getAdditionalData());

        // Set updated status to Local transaction
        $transactionData['status'] = $response['status'];
        $order->getPayment()->setAdditionalData(serialize($transactionData));
        if ($response['status'] == BinaryPay::STATUS_AUTHORISED) {
            // Change the order's status to processing
            $order->setState(Mage_Sales_Model_Order::STATE_PROCESSING, true)->save();
        }

        return $this;
    }

    /**
     * _prepareFetchData  - Validate Timeout and Increment before try to fetch data from local or API
     * @return
     */
    protected function _prepareFetchData()
    {
        $session        = Mage::getSingleton('core/session');
        $counter        = $session->getOeCount();
        $incrementId    = $session->getIncrementId();

        if ($counter > 20) {
            $message = $this->__('Timeout! Your payment might be failed. Please go check your payment status or contact us.');
            Mage::throwException($message);
        }

        if (!$incrementId) {
            $message = $this->__('You do not have permission to access the page.');
            Mage::throwException($message);
        }

        return $this;
    }

    public function startAction()
    {
        $layout = $this->getLayout();
        // Since this page is constantly refreshing, we have to keep it mininmal.

        $this->loadLayout();
        $update = $layout->getUpdate();
        $update->addHandle('cms');

        $block = $this->getLayout()->createBlock('core/template');
        $block->setTemplate('magebinary/binarypay/onlineeftpos/start.phtml');

        $page = $layout->getBlock('root');
        $page->setTemplate('page/1column.phtml');

        $header = $page->unsetChild('header');

        $page->unsetChild('footer');
        $layout->getBlock('content')->append($block);

        $this->renderLayout();
        return $this;
    }

    /**
     * returnAction - the controller to handle the response that sent by paymark Api
     */
    public function returnAction()
    {
        $response = $this->getRequest()->getParams();

        if (!isset($response) || empty($response)) {
            $this->_redirect('/');
            return;
        }

        try {
            // Validate post response from paymark Api
            $this->_validateCallback();

            // Wait Magento create order
            sleep(5);

            $this->_processOrder($response);

            // Stop trying to fetch data from api or local
            $result = array(
                'status'    => $response['status']
            );

            // Unset session before redirect back to success page
            $session->unsIncrementId()
                    ->unsOeCount();
        } catch (BinaryPay_Exception $e) {
            $this->_handleExceptions($e);
        } catch (Exception $e){
            $this->_handleExceptions($e, true);
        }

        $this->_sendMessage($result);
        return;
    }

    public function checkAction()
    {
        $session = Mage::getSingleton('core/session');
        $counter = $session->getOeCount();
        empty($counter) ? $session->setOeCount(1) : $session->setOeCount($counter + 1);

        try {
            // Validate timeout and increment id
            $this->_prepareFetchData();

            if ($counter <= 10) {
                // Fetch local transaction detail
                $this->_checkLocalTranscationInfo();
            } else {
                // Fetch API transaction detail
                $this->_checkApiTranscationInfo();
            }
        } catch (BinaryPay_Exception $e) {
            $this->_handleExceptions($e);
        } catch (Exception $e) {
            $this->_handleExceptions($e, true);
        }

        return $this;
    }

    protected function _retrieveTransactionById($id)
    {
        $gateway = Mage::getModel('magebinary_binarypay/onlineeftpos')->getGateway();
        return $gateway->getTransaction($id);
    }

    protected function _checkLocalTranscationInfo()
    {
        $result         = array();
        $session        = Mage::getSingleton('core/session');
        $incrementId    = $session->getIncrementId();
        // Get Transaction Data if increment id has passed in
        $order = Mage::getModel('sales/order')->load($incrementId, 'increment_id');

        if (!$order) {
            $session->unsIncrementId()->unsOeCount();
            $message = $this->__('No orders has been created.');
            Mage::throwException($message);
        }

        $transaction = unserialize($order->getPayment()->getAdditionalData());
        if ($transaction['status'] === BinaryPay::STATUS_AUTHORISED) {
            // Prevent session abuse after all operations
            $session->unsIncrementId()->unsOeCount();

            $result = array(
                'status'    => $transaction['status']
            );
        }

        $this->_sendMessage($result);
        return;
    }

    protected function _checkApiTranscationInfo()
    {
        $session        = Mage::getSingleton('core/session');
        $incrementId    = $session->getIncrementId();

        // Get Transaction Data if increment id has passed in
        $order      = Mage::getModel('sales/order')->load($incrementId, 'increment_id');

        if (!$order) {
            $session->unsIncrementId()->unsOeCount();
            $message = $this->__('No orders has been created.');
            Mage::throwException($message);
        }

        $transactionId  = unserialize($order->getPayment()->getAdditionalData())['id'];

        // Get transaction data from API
        $transaction    = $this->_retrieveTransactionById($transactionId);


        if (empty($transaction)) {
            Mage::throwException('Your payment information does not exists in the Paymark\'s system. Please contact the website administrator for detail information.');
        }

        try {
            // Update local transaction data and order status
            $this->_updateTransaction($transaction);
        } catch (BinaryPay_Exception $e) {
            $this->_handleExceptions($e);
        } catch (Exception $e){
            $this->_handleExceptions($e, true);
        }
        return $this;
    }


    protected function _validateResponse($response)
    {
        $session      = Mage::getSingleton('core/session');
        $gateway      = Mage::getModel('magebinary_binarypay/onlineeftpos')->getGateway();
        $status       = is_array($response) ? $response['status'] : $response;

        $transaction  = $gateway->arrayFlatten($response);

        if (!isset($status)) {
            $message = $this->__('No status returned. Please contact us.');

            // Prevent session abuse after all operations
            $session->unsIncrementId()->unsOeCount();
            Mage::throwException($message);
        }

        if (!isset($transaction['orderId'])) {
            $message = $this->__('No matched order record. Please contact us.');

            // Prevent session abuse after all operations
            $session->unsIncrementId()->unsOeCount();
            Mage::throwException($message);
        }
        //     // Prevent user update cart after redirecting
        //     // Check Amount and Quote Amount
        // if (!isset($response['amount']) || $response['amount'] != $quote->getGrandTotal()) {
        //     Mage::throwException($this->__('The order and payment amount cannot be matched up correctly.'));
        // }

        if ($status === BinaryPay::STATUS_SUBMITTED) {
            // Waiting Paymark API's latest response
        } else {
            $order = Mage::getModel('sales/order')->load($transaction['orderId'], 'increment_id');

            // Status updated and is not a successfull payment
            // Save new local transaction into local
            $order->getPayment()->setAdditionalData(serialize($transaction));
            $order->save();

            // Display error message to the user when purchase failed
            $message = $this->__("Purchase failed. The payment has been $status");

            // Clean other error in the session
            $this->_getCheckoutSession()->setMessage('');

            // Prevent session abuse after all operations
            $session->unsIncrementId()->unsOeCount();

            // Display Error
            $this->_getCheckoutSession()->addError($message);

            $result = array(
                'status' => $status
            );
        }

        $this->_sendMessage($result);
        return;
    }
    /**
     * _updateTransaction - update local transaction data and order status
     * @param  [array,string] $response
     * @return $this
     */
    protected function _updateTransaction($response)
    {
        $session            = Mage::getSingleton('core/session');
        $incrementId        = $session->getIncrementId();
        $order              = Mage::getModel('sales/order')->load($incrementId, 'increment_id');

        $localTransaction   = unserialize($order->getPayment()->getAdditionalData());
        $status             = isset($response['status']) ? $response['status'] : BinaryPay::STATUS_UNKNOWN;
        try {
            // Update locaWl transaction status with response status
            $localTransaction['status']   = $status;

            // Only change order status to processing when response status shows as AUTHORISED
            if ($status === BinaryPay::STATUS_AUTHORISED) {

                // Save new local transaction into local
                $order->getPayment()->setAdditionalData(serialize($localTransaction));

                // Update Order Status
                $order->setState(Mage_Sales_Model_Order::STATE_PROCESSING, true)->save();

                // Prevent session abuse after all operations
                $session->unsIncrementId()->unsOeCount();

                $result = array(
                    'status'    => $status
                );
            } else {
                $this->_validateResponse($response);
            }

        } catch (BinaryPay_Exception $e) {
            $this->_handleExceptions($e);
        } catch (Exception $e){
            $this->_handleExceptions($e, true);
        }

        $this->_sendMessage($result);
        return;
    }

    protected function _sendMessage($message)
    {
        if (isset($message) && !empty($message)) {
            $this->getResponse()->clearHeaders()->setHeader('Content-type', 'application/json', true);
            $this->getResponse()->setBody(json_encode($message));
        }

        return $this;
    }

    protected function _getCheckoutSession()
    {
        if (!$this->_checkoutSession) {
            $this->_checkoutSession = Mage::getSingleton('checkout/session');
        }
        return $this->_checkoutSession;
    }

    protected function _handleExceptions($exception, $logger = false)
    {
        $session = Mage::getSingleton('core/session');

        // Prevent session abuse
        $session->unsIncrementId()
                ->unsOeCount();

        $result = array(
            'message' => $exception->getMessage()
        );

        // Clean other errors in the session
        $this->_getCheckoutSession()->getMessage(true);

        // Display current exception
        $this->_getCheckoutSession()->addError($exception->getMessage());

        if ($logger) {
            Mage::log($exception->getMessage(), Zend_Log::ERR, 'paymark.log', true);
        }

        $this->_sendMessage($result);
        return;
    }
}