<?php
/**
 * Magento BinaryPay Payment Extension
 *
 * NOTICE OF LICENSE
 *
 * Copyright 2017 - 2019 MageBinary
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @category    MageBinary
 * @package     MageBinary_BinaryPay
 * @author      MageBinary Team
 * @copyright   Copyright (c) 2017 - 2019 MageBinary (http://www.magebinary.com)
 * @license     http://www.apache.org/licenses/LICENSE-2.0
 */
class MageBinary_BinaryPay_PolipayController extends MageBinary_BinaryPay_Controller_Abstract
{
    /**
     * @var bool
     */
    protected $_debug = true;

    /**
     * $_successStatuses
     * @var array
     */
    protected $_successStatuses = [
        MageBinary_BinaryPay_Model_Method_Polipay::SUCCESSFUL
    ];

    /**
     * $_pendingStatuses
     * @var array
     */
    protected $_pendingStatuses = [
        MageBinary_BinaryPay_Model_Method_Polipay::RECEIPTUNVERIFIED,
        MageBinary_BinaryPay_Model_Method_Polipay::UNKNOWN
    ];

    public function returnAction()
    {
        $session = $this->_getCheckoutSession();

        try {
            $this->_apiResponse = $response = $this->getRequest()->getQuery();

            $token = $response['token'];

            // Get transaction status by using the data that sent by Popipay GetTransaction API
            //AS we using Retrive in GetTransaction API we no need end point with "Nudge" where using Notification URL
            $response    = $this->_retrieveResponseByToken($response);
            //Added token into Response
            $response['token'] = $token;

            // Pass Response To Capture
            $session->setResponse($response);

            if ($this->_debug) {
                Mage::log('RESPONSE FROM POLIPAY', Zend_Log::DEBUG, 'BinaryPay.log', true);
                Mage::log(json_encode($response), Zend_Log::DEBUG, 'BinaryPay.log', true);
                Mage::log('END OF RESPONSE', Zend_Log::DEBUG, 'BinaryPay.log', true);
            }

            $this->_validateResponse($response);

            if ($this->isBackendOrder()) {
                // $this->createOrderForBackend($quote);
            }

            $this->placeOrder();

        } catch (BinaryPay_Exception $e) {
            $this->addErrorException($session, $e);
            $this->_redirectUrl = MageBinary_BinaryPay_Model_Config::CHECKOUT_CART_PAGE_URL;
            Mage::logException($e);
        } catch (Mage_Core_Exception $e) {
            $this->addErrorException($session, $e);
            $this->placeOrder($e);
            Mage::logException($e);
        } catch (Exception $e) {
            $this->addErrorException($session, $e);
            $this->placeOrder($e);
            Mage::logException($e);
        }

        $session->unsIsPending();
        $session->unsResponse();
        $session->unsPaymentStatus();

        return $this->_redirect($this->_redirectUrl);
    }

    /**
     * Validate Response from Paymark webpayment
     * @param  Array $request - Response From API
     * @return Boolean
     */
    protected function _validateResponse($response)
    {
        $session    = $this->_getCheckoutSession();
        $quote      = $this->_getQuote();
        $purchaseToken = $session->getPolipayToken();
        $reference = $session->getPolipayReference();

        // Prevent user reuse the session
        $session->unsetData('poliapay_token');
        $session->unsetData('poliapay_refernece');

        // Check return response
        // Check request types & origin
        if (!$this->getRequest()->isGet()) {
            $session->setIsPending(true);
            Mage::throwException($this->__('Warning: The response is not a GET request'));
        }
        // Prevent user update cart after redirecting
        // Check Amount and Quote Amount
        if (!isset($response['PaymentAmount']) || $response['PaymentAmount'] != $quote->getGrandTotal()) {
            Mage::throwException($this->__('The order and payment amount cannot be matched up correctly.'));
        }

        /* This step is avoided when admin is placing the order*/
        // Check with API & Transcation particular
        if (!isset($response['token']) || !$purchaseToken || $purchaseToken != $response['token']) {
            $session->setIsPending(true);
            Mage::throwException($this->__('The purchase token cannot be matched up with the response.'));
        }

        if (!isset($response['MerchantReference']) || !$reference || $reference != $response['MerchantReference']) {
            $session->setIsPending(true);
            Mage::throwException($this->__('The reference cannot be matched up with the response.'));
        }

        $status = $response['TransactionStatus'];
        $session->setPaymentStatus($status);

        // Not paid or pending payment status then redirect customer to cart page



        if (!$this->isPaid($status) && !$this->isPending($status)) {
            throw new BinaryPay_Exception($this->translateErrorMessages($response));
        }

        if ($this->isPending($status)) {
            Mage::throwException('The payment response has been received but with an UNKNOWN status.');
        }

        return $this;
    }

    public function translateErrorMessages($response)
    {
        $status = $response['TransactionStatus'];
        switch ($status) {
            case MageBinary_BinaryPay_Model_Method_Polipay::FAILED:
                $message = $this->__($response['ErrorMessage']);
                break;
            default:
                $message = $this->__($response['TransactionStatus']);
                break;
        }
        return $message;
    }


    protected function _retrieveResponseByToken($response)
    {
        $api     = Mage::getModel('magebinary_binarypay/polipay');
        $gateway = $api->getGateway();

        $request = array(
            BinaryPay::PURCHASE_TOKEN => $response['token']
        );
        if (empty($request[BinaryPay::PURCHASE_TOKEN])) {
            Mage::throwException('The purchase token not received');
        }

        return $gateway->retrieve($request);
    }
}