<?php
/**
 * Magento BinaryPay Payment Extension
 *
 * NOTICE OF LICENSE
 *
 * Copyright 2017 MageBinary
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @category    MageBinary
 * @package     MageBinary_BinaryPay
 * @author      MageBinary Team
 * @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
 * @license     http://www.apache.org/licenses/LICENSE-2.0
 */
class MageBinary_BinaryPay_QcardController extends MageBinary_BinaryPay_Controller_Abstract
{
    /**
     * @var Mage_Sales_Model_Quote
     */
    protected $_quote = false;

    /**
     * @var array
     */
    protected $_apiResponse;

    /**
     * @var bool
     */
    protected $_debug = true;

    /**
     * returnAction
     *
     * The return Url for Qcard response
     * @return $this
     */
    public function returnAction()
    {
        try {
            $session        = $this->_getCheckoutSession();
            $request        = $this->getRequest();

            // Qcard send back process number and merchant transaction id via GET method
            $processNo      = $request->getParam('process_no');
            $transactionId  = $request->getParam('merchant_transaction_id');

            // Gather the information that will need to send to get the payment's status
            $transaction = array(
                'merchant_transaction_id'   => $transactionId,
                'process_no'                => $processNo
            );

            // Get payment status by using the data that sent by Qcard Api
            $response    = $this->_retrieveTransactionById($transaction);
            $response    = array_merge($response, array('processNo' => $processNo));

            // Save the response into the session
            $session->setQcardResponse($response);
            $this->_apiResponse = $response;

            if ($this->_debug) {
                Mage::log($response, null, 'Qcard.log', true);
            }

            if (!$this->_isValidResponse()) {
                Mage::throwException('Oops! We got an error from the payment gateway. Please try again later or contact us for the details.');
            }

            if ($this->_isPaymentSuccess()) {

                if ($this->_isPaymentInProgress()) {
                    $isPending = true;
                    $session->setIsPending($isPending);
                }

                return $this->_forward('placeOrder');
            }

            Mage::throwException($this->_getPaymentErrorMessages());

        } catch (BinaryPay_Exception $e) {
            $session->addError($e->getMessage());
        } catch (Mage_Core_Exception $e) {
            $session->addError($e->getMessage());
        } catch (Exception $e) {
            $session->addError($e->getMessage());
            Mage::log($e->getTrace(), null, 'Qcard.log', true);
        }

        return $this->_redirect('checkout/cart');
    }

    public function placeOrderAction()
    {
        $quote = $this->_getQuote();
        $session = $this->_getCheckoutSession();

        try {
            if ($this->isBackendOrder()) {
                $this->createOrderForBackend($quote);
            }

            $this->placeOrder();
            $this->clearShoppingCart();

        }  catch (BinaryPay_Exception $e) {
            Mage::helper('checkout')->sendPaymentFailedEmail(
                $this->_getQuote(),
                $e->getMessage()
            );
            $session->addError($e->getMessage());
        } catch (Exception $e) {
            Mage::helper('checkout')->sendPaymentFailedEmail(
                $this->_getQuote(),
                $e->getMessage()
            );
            $session->addError($e->getMessage());
            Mage::log($e->getTrace(), Zend_Log::ERR, 'BinaryPay.log', true);
        }
        return $this->_redirect('checkout/onepage/success');
    }

    protected function _isPaymentOnHold()
    {
        return $this->_getCheckoutSession()->getIsPending();
    }

    protected function _isPaymentInProgress()
    {
        return $this->_getResponseStatus() == MageBinary_BinaryPay_Model_Method_Flexilong::IN_PROGRESS;
    }

    protected function _isPaymentSuccess()
    {
        return $this->_getResponseStatus() == MageBinary_BinaryPay_Model_Method_Flexilong::APPROVED
                || $this->_getResponseStatus() == MageBinary_BinaryPay_Model_Method_Flexilong::IN_PROGRESS;
    }

    protected function _getResponseStatus()
    {
        $response = $this->_apiResponse;
        return $response['paymentStatus']['statusDesc'];
    }

    protected function _getPaymentErrorMessages()
    {
        switch ($this->_getResponseStatus()) {
            case MageBinary_BinaryPay_Model_Method_Flexilong::CANCEL:
                $message = $this->__('You have canceled the payment at checkout. Please contact us for more assistance or select another payment method to complete checkout.');
                break;
            case MageBinary_BinaryPay_Model_Method_Flexilong::ERROR:
            case MageBinary_BinaryPay_Model_Method_Flexilong::DECLINED:
                $message = $this->__('Payment failed! If you want further information, contact us for details.');
                break;
            default:
                $message = $this->__('Unknown error has been detected Please contact us for more assistance.');
                break;
        }
        return $message;
    }

    protected function _isValidResponse()
    {
        $response = $this->_apiResponse;
        if (isset($response['paymentStatus']['statusDesc'])) {
            return true;
        }
        return false;
    }

    /**
     * _retrieveTransactionById
     *
     * Get payment status by send POST request to Qcard API
     * @param  array $transaction
     * @return array
     */
    protected function _retrieveTransactionById($transaction)
    {
        $api     = Mage::getModel('magebinary_binarypay/flexilong');
        $gateway = $api->getGateway();
        $request = array(
            BinaryPay::API_KEY        => $api->getPaymentMethodConfig('qcard_status_apikey', true),
            BinaryPay::TRANSACTION_ID => $transaction['merchant_transaction_id'],
            BinaryPay::PROCESS_NO     => $transaction['process_no']
        );

        return $gateway->getTransaction($request);
    }
}